import datetime

from django.utils.timezone import now

from post_office.mail import send_queued
from post_office.utils import cleanup_expired_mails

from .settings import get_celery_enabled

# Only define the tasks and handler if we can import celery.
# This allows the module to be imported in environments without Celery, for
# example by other task queue systems such as Huey, which use the same pattern
# of auto-discovering tasks in "tasks" submodules

try:
    if get_celery_enabled():
        from celery import shared_task
    else:
        raise NotImplementedError()
except (ImportError, NotImplementedError):
    pass
else:
    @shared_task(ignore_result=True)
    def send_queued_mail(*args, **kwargs):
        send_queued()

    def queued_mail_handler(sender, **kwargs):
        """
        To be called by post_office.signals.email_queued.send()
        """
        send_queued_mail.delay()

    @shared_task(ignore_result=True)
    def cleanup_mail(*args, **kwargs):
        days = kwargs.get('days', 90)
        cutoff_date = now() - datetime.timedelta(days)
        delete_attachments = kwargs.get('delete_attachments', True)
        cleanup_expired_mails(cutoff_date, delete_attachments)



from collections import OrderedDict
from email.mime.base import MIMEBase
from django.core.files.base import ContentFile
from django.core.mail.backends.base import BaseEmailBackend

from .settings import get_default_priority


class EmailBackend(BaseEmailBackend):

    def open(self):
        pass

    def close(self):
        pass

    def send_messages(self, email_messages):
        """
        Queue one or more EmailMessage objects and returns the number of
        email messages sent.
        """
        from .mail import create
        from .utils import create_attachments

        if not email_messages:
            return

        for email_message in email_messages:
            subject = email_message.subject
            from_email = email_message.from_email
            headers = email_message.extra_headers
            message = email_message.message()

            # Look for first 'text/plain' and 'text/html' alternative in email
            plaintext_body = html_body = ''
            for part in message.walk():
                if part.get_content_type() == 'text/plain':
                    plaintext_body = part.get_payload()
                    if html_body:
                        break
                if part.get_content_type() == 'text/html':
                    html_body = part.get_payload()
                    if plaintext_body:
                        break

            attachment_files = {}
            for attachment in email_message.attachments:
                if isinstance(attachment, MIMEBase):
                    attachment_files[attachment.get_filename()] = {
                        'file': ContentFile(attachment.get_payload()),
                        'mimetype': attachment.get_content_type(),
                        'headers': OrderedDict(attachment.items()),
                    }
                else:
                    attachment_files[attachment[0]] = ContentFile(attachment[1])

            email = create(sender=from_email,
                           recipients=email_message.to, cc=email_message.cc,
                           bcc=email_message.bcc, subject=subject,
                           message=plaintext_body, html_message=html_body,
                           headers=headers)

            if attachment_files:
                attachments = create_attachments(attachment_files)

                email.attachments.add(*attachments)

            if get_default_priority() == 'now':
                email.dispatch()



from django.utils.html import mark_safe, format_html
from django.utils.translation import gettext_lazy

try:
    import bleach

    clean_html = lambda body: mark_safe(bleach.clean(
        body,
        tags=[
            'a',
            'abbr',
            'acronym',
            'b',
            'blockquote',
            'br',
            'caption',
            'center',
            'code',
            'em',
            'div',
            'font',
            'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
            'head',
            'hr',
            'i',
            'img',
            'label',
            'li',
            'ol',
            'p',
            'pre',
            'span',
            'strong',
            'table', 'tbody', 'tfoot', 'td', 'th', 'thead', 'tr',
            'u',
            'ul',
        ],
        attributes={
            'a': ['class', 'href', 'id', 'style', 'target'],
            'abbr': ['class', 'id', 'style'],
            'acronym': ['class', 'id', 'style'],
            'b': ['class', 'id', 'style'],
            'blockquote': ['class', 'id', 'style'],
            'br': ['class', 'id', 'style'],
            'caption': ['class', 'id', 'style'],
            'center': ['class', 'id', 'style'],
            'code': ['class', 'id', 'style'],
            'em': ['class', 'id', 'style'],
            'div': ['class', 'id', 'style', 'align', 'dir'],
            'font': ['class', 'id', 'style', 'color', 'face', 'size'],
            'h1': ['class', 'id', 'style', 'align', 'dir'],
            'h2': ['class', 'id', 'style', 'align', 'dir'],
            'h3': ['class', 'id', 'style', 'align', 'dir'],
            'h4': ['class', 'id', 'style', 'align', 'dir'],
            'h5': ['class', 'id', 'style', 'align', 'dir'],
            'h6': ['class', 'id', 'style', 'align', 'dir'],
            'head': ['dir', 'lang'],
            'hr': ['align', 'size', 'width'],
            'i': ['class', 'id', 'style'],
            'img': ['class', 'id', 'style', 'align', 'border', 'height', 'hspace', 'src', 'usemap', 'vspace', 'width'],
            'label': ['class', 'id', 'style'],
            'li': ['class', 'id', 'style', 'dir', 'type'],
            'ol': ['class', 'id', 'style', 'dir', 'type'],
            'p': ['class', 'id', 'style', 'align', 'dir'],
            'pre': ['class', 'id', 'style'],
            'span': ['class', 'id', 'style'],
            'strong': ['class', 'id', 'style'],
            'table': ['class', 'id', 'style', 'align', 'bgcolor', 'border', 'cellpadding', 'cellspacing', 'dir', 'frame', 'rules', 'width'],
            'tbody': ['class', 'id', 'style'],
            'tfoot': ['class', 'id', 'style'],
            'td': ['class', 'id', 'style', 'abbr', 'align', 'bgcolor', 'colspan', 'dir', 'height', 'lang', 'rowspan', 'scope', 'style', 'valign', 'width'],
            'th': ['class', 'id', 'style', 'abbr', 'align', 'background', 'bgcolor', 'colspan', 'dir', 'height', 'lang', 'scope', 'style', 'valign', 'width'],
            'thead': ['class', 'id', 'style'],
            'tr': ['class', 'id', 'style', 'align', 'bgcolor', 'dir', 'style', 'valign'],
            'u': ['class', 'id', 'style'],
            'ul': ['class', 'id', 'style', 'dir', 'type'],
        },
        strip=True,
        strip_comments=True,
        styles=[
            'border', 'border-top', 'border-right', 'border-bottom', 'border-left',
            'border-radius',
            'box-shadow',
            'height',
            'margin', 'margin-top', 'margin-right', 'margin-bottom', 'margin-left',
            'padding', 'padding-top', 'padding-right', 'padding-bottom', 'padding-left',
            'width',
            'max-width',
            'min-width',
            'border-collapse',
            'border-spacing',
            'caption-side',
            'empty-cells',
            'table-layout',
            'direction',
            'font',
            'font-family',
            'font-style',
            'font-variant',
            'font-size',
            'font-weight',
            'letter-spacing',
            'line-height',
            'text-align',
            'text-decoration',
            'text-indent',
            'text-overflow',
            'text-shadow',
            'text-transform',
            'white-space',
            'word-spacing',
            'word-wrap',
            'vertical-align',
            'color',
            'background',
            'background-color',
            'background-image',
            'background-position',
            'background-repeat',
            'bottom',
            'clear',
            'cursor',
            'display',
            'float',
            'left',
            'opacity',
            'outline',
            'overflow',
            'position',
            'resize',
            'right',
            'top',
            'visibility',
            'z-index',
            'list-style-position',
            'list-style-tyle',
        ],
    ))
except ImportError:
    # if bleach is not installed, render HTML as escaped text to prevent XSS attacks
    heading = gettext_lazy("Install 'bleach' to render HTML properly.")
    clean_html = lambda body: format_html('<p><em>{heading}</em></p>\n<div>{body}</div>',
                                          heading=heading, body=body)
